<?php

namespace Drupal\xray\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\PagerSelectExtender;
use Drupal\Core\Database\Query\TableSortExtender;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for xray routes.
 */
class XrayController extends ControllerBase {

  /**
   * The database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database')
    );
  }

  /**
   * Constructs a XrayController object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   A database connection.
   */
  public function __construct(Connection $database) {
    $this->database = $database;
  }


  /**
   * Displays an overview of xray reports.
   *
   * @return array
   *   A render array as expected by
   *   \Drupal\Core\Render\RendererInterface::render().
   */
  public function overview() {

    $build['xray_overview'] = [
      '#type' => 'link',
      '#title' => "View mode analysis",
      '#url' => new Url('xray.view_mode_analysis'),
    ];

    return $build;
  }

  /**
   * Displays a report of view mode usage.
   *
   * @return array
   *   A render array as expected by
   *   \Drupal\Core\Render\RendererInterface::render().
   */
  public function view_mode_analysis() {
    $header = [
      [
        'data' => $this->t('View mode'),
        'field' => 'x.view_mode',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Entity type'),
        'field' => 'x.entity_type',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Bundle'),
        'field' => 'x.bundle',
        'class' => [RESPONSIVE_PRIORITY_MEDIUM],
      ],
      [
        'data' => $this->t('Occurrences'),
        'class' => [RESPONSIVE_PRIORITY_LOW],
      ],
    ];

    $query = $this->database->select('xray_view_mode_analysis', 'x')
      ->extend(PagerSelectExtender::class)
      ->extend(TableSortExtender::class);
    $query->fields('x', [
      'xvmaid',
      'view_mode',
      'entity_type',
      'bundle',
      'occurrences'
    ]);
    $result = $query
      ->limit(100)
      ->orderByHeader($header)
      ->execute();

    foreach($result as $xvm_item) {
      $rows[] = [
        'data' => [
          // Cells.
          $xvm_item->view_mode,
          $xvm_item->entity_type,
          $xvm_item->bundle,
          $xvm_item->occurrences
        ]
      ];
    }

    $build['xray_view_mode_analysis_table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#attributes' => ['id' => 'admin-xray-view-mode', 'class' => ['admin-xray-view-mode']],
      '#empty' => $this->t('No report available.'),
    ];
    $build['xray_pager'] = ['#type' => 'pager'];
    return $build;
  }
};
